/**
 ** Copyright (c) 2002 Sun Microsystems, Inc.
 **
 ** See the file "license.terms" for information on usage and redistribution
 ** of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 **/

/**
 **  gcspy_command_stream.cpp
 **
 **  Facilities for buffered commands
 **/

#include <stdio.h>

#include "gcspy_command_stream.hpp"

gcspyCommandStream::gcspyCommandStream (gcspy_command_t **cmds,
					gcspy_length_t len) {
  _cmds = cmds;
  _len = len;
  //  _output = NULL;
}

gcspyCommandStream::gcspyCommandStream (gcspy_length_t len) {
  _cmds = new gcspy_command_t*[len];
  if (_cmds == NULL) {
    char buffer[128];
    sprintf(buffer, "new failed for the command array of length %u", len);
    gcspy_raise_error(buffer);
  }
  _len = len;
  for (int i = 0; i < len; ++i)
    _cmds[i] = NULL;
  //  _output = NULL;
}

void
gcspyCommandStream::execute (gcspyBufferedInput *input, void *aux) {
  int magic = input->read_int();
  gcspy_assert(magic == magic_start_tag);

  int tag = (int) get_tag(input);
  while (tag != end_tag) {
    gcspy_assert((tag >= 0) && (tag < _len));
    gcspy_assert(_cmds[tag] != NULL);
    _cmds[tag](input, aux);

    tag = (int) get_tag(input);
  }
  magic = (int) input->read_int();
  gcspy_assert(magic == magic_end_tag);
  input->close();
}
