/**
 ** Copyright (c) 2002 Sun Microsystems, Inc.
 **
 ** See the file "license.terms" for information on usage and redistribution
 ** of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 **/

/**
 **  gcspy_test_driver.cpp
 **
 **  Simple test driver
 **/

#include <stdio.h>

#include "test_driver.hpp"

void
testDriver::reset (gcspy_length_t tile_num) {
  if (tile_num != get_tile_num()) {
    resize(tile_num);
    setup_tile_names();
    _used_space_data = _used_space_stream->get_int_data();
    _card_table_data = _card_table_stream->get_byte_data();
    _object_data = _object_stream->get_int_data();
  }

  reset_data();
  _used_space_summary[1] = get_tile_num() * 512;
  _card_table_summary[card_state_clean] = get_tile_num();

  set_control(1, control_separator);
  set_control_range(10, 10, control_link);
  set_control_range(get_tile_num() - 20, 20, control_unused);
  set_control_range(50, 10, control_background);

  char buffer[128];
  sprintf(buffer, "Tile Num: %d\n", get_tile_num());
  set_space_info(buffer);
}

void
testDriver::object (gcspy_index_t index) {
  _used_space_data[index] += 64;
  _used_space_summary[0] += 64;

  _object_data[index] += 1;
  _object_summary[0] += 1;
}

void
testDriver::card (gcspy_index_t index, char state) {
  _card_table_data[index] = state;

  if (state != card_state_clean) {
    _card_table_summary[state] += 1;
    _card_table_summary[card_state_clean] -= 1;
  }
}

testDriver::testDriver (const char      *name,
			gcspy_length_t   tile_num,
			gcspy_bool_t     main_space) 
  : gcspySpace(name, "Simple Driver", "Block: ", "",
	       tile_num, NULL, main_space) {

  _used_space_stream = new gcspyStream(gcspyStream::int_type,
				       "Used Space",
				       0, 512, 0, 0,
				       "Used Space: ", " bytes",
				       gcspyStream::presentation_percent,
				       gcspyStream::paint_style_zero,
				       0, gcspyColorDB::get_color("Red"), 1);
  add_stream(_used_space_stream);
  _used_space_data = _used_space_stream->get_int_data();
  _used_space_summary = _used_space_stream->get_summary();

  _card_table_stream = new gcspyStream(gcspyStream::byte_type,
				       "Card Table",
				       card_state_dirty, card_state_clean,
				       card_state_clean, card_state_clean,
				       "Card Table: ", "",
				       gcspyStream::presentation_enum,
				       gcspyStream::paint_style_plain,
				       0, gcspyColorDB::get_color("Off White"), 1);
  _card_table_stream->add_enum_name(card_state_dirty,   "DIRTY");
  _card_table_stream->add_enum_name(card_state_younger, "YOUNGER");
  _card_table_stream->add_enum_name(card_state_clean,   "CLEAN");
  add_stream(_card_table_stream);
  _card_table_data = _card_table_stream->get_byte_data();
  _card_table_summary = _card_table_stream->get_summary();

  _object_stream = new gcspyStream(gcspyStream::int_type,
				   "Objects",
				   0, 64, 0, 0,
				   "Objects: ", "",
				   gcspyStream::presentation_plus,
				   gcspyStream::paint_style_zero,
				   0, gcspyColorDB::get_color("Yellow"), 1);
  add_stream(_object_stream);
  _object_data = _object_stream->get_int_data();
  _object_summary = _object_stream->get_summary();

  setup_tile_names();
}

void
testDriver::setup_tile_names () {
  for (int i = 0; i < get_tile_num(); ++i) {
    char buffer[128];
    sprintf(buffer, "   [%d,%d)", i, i+1);
    set_tile_name(i, buffer);
  }
}
