/**
 ** $id$
 ** Copyright (c) 2002 Sun Microsystems, Inc.
 **
 ** See the file "license.terms" for information on usage and redistribution
 ** of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 **/

package gcspy;

import gcspy.utils.Arguments;
import gcspy.vis.MainFrame;

import java.util.logging.Level;
import java.util.logging.Logger;

import javax.swing.SwingUtilities;
import javax.swing.UIManager;

/**
 * Main class of GCspy
 * 
 * @author Tony Printezis
 */
public class Main {
	
	private static final Logger logger = Logger.getLogger(Main.class.getName());

	/**
	 * Print usage messages
	 * 
	 * @param mesg
	 *            An error message
	 */
	static private void usage(String mesg) {
		System.out.println();
		System.out.println("usage:");
		System.out.println("    gcspy.Main <options>");
		System.out.println();
		System.out.println("where <options> are");
		System.out.println("    -server <host> <port>");
		System.out.println("        specifies where the server is");
		System.out.println("        e.g. -server islay 3000");
		System.out.println("    -connect (requires -server)");
		System.out.println("        connects immediately");
		System.out.println("    -pause");
		System.out.println("        pauses immediately after it connects");
		System.out.println();
		if (mesg != null) {
			System.out.println("Error:");
			System.out.println("    " + mesg);
			System.out.println();
		}

		System.exit(-1);
	}

	/** GCspy main entry point */
	static public void main(String args[]) {
		Arguments arguments = new Arguments();

		int serverParams[] = { Arguments.STRING_TYPE, Arguments.POS_INT_TYPE };
		Object serverValues[] = { "localhost", 3000 };
		arguments.add("-server", false, serverParams, serverValues);
		String connectRequires[] = { "-server" };
		arguments.add("-connect", connectRequires, null);
		String pauseRequires[] = { "-server" };
		arguments.add("-pause", pauseRequires, null);

		if (!arguments.parse(args))
			usage(arguments.getError());

		Object values[] = arguments.getValues("-server");
		final String host = (String) values[0];
		final int port = (Integer) values[1];
		final boolean connect = arguments.isSet("-connect");
		final boolean pause = arguments.isSet("-pause");

		SwingUtilities.invokeLater(new Runnable() {
			@Override
			public void run() {
				try {
					UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
				} catch (Exception e) {
					logger.log(Level.WARNING, "Could not set system look and feel, using default", e);
				}

				MainFrame frame = new MainFrame(host, port, pause, connect);
				frame.setSize(1024, 768);
				frame.setVisible(true);
			}
		});
	}

}
