package gcspy.interpreter;

public class StreamData {

	public static enum Datatype {
		Byte, /** Data type is byte */
		Short, /** Data type is short */
		Int, /** Data type is int */
		Unknown;
		/** Data type is unknown yet */

		public static Datatype map(int index) {
			return Datatype.values()[index];
		}
	}

	private Datatype dataType = Datatype.Unknown;
	private byte bData[];
	private short sData[];
	private int iData[];

	public StreamData(Datatype type) {
		setType(type);
	}

	public StreamData() {
		this(Datatype.Unknown);
	}

	public StreamData(byte[] bs) {
		this(Datatype.Byte);
		setData(bs);
	}

	public StreamData(short[] s) {
		this(Datatype.Short);
		setData(s);
	}

	public StreamData(int[] is) {
		this(Datatype.Int);
		setData(is);
	}

	public void setType(Datatype type) {
		if (getType() != type && getType() != Datatype.Unknown) {
			throw new Error("Cannot set datatype twice");
		}
		if (type == Datatype.Unknown) {
			throw new Error("Cannot set to unknown");
		}
		this.dataType = type;
	}

	public Datatype getType() {
		return this.dataType;
	}

	public byte[] getBytes() {
		if (getType() != Datatype.Byte) {
			throw new Error("Illegal access");
		}
		return this.bData;
	}

	public short[] getShorts() {
		if (getType() != Datatype.Short) {
			throw new Error("Illegal access");
		}
		return this.sData;
	}

	public int[] getInts() {
		if (getType() != Datatype.Int) {
			throw new Error("Illegal access");
		}
		return this.iData;
	}

	public void setData(byte[] data) {
		if (getType() != Datatype.Byte) {
			throw new Error("Setting data for wrong type");
		}
		this.bData = data;
	}

	public void setData(short[] data) {
		if (getType() != Datatype.Short) {
			throw new Error("Setting data for wrong type");
		}
		this.sData = data;
	}

	public void setData(int[] data) {
		if (getType() != Datatype.Int) {
			throw new Error("Setting data for wrong type");
		}
		this.iData = data;
	}

	public int size() {
		switch (getType()) {
		case Byte:
			return getBytes().length;
		case Short:
			return getShorts().length;
		case Int:
			return getInts().length;
		default:
			throw new Error("Invalid data type");
		}

	}

}
