package gcspy.vis;

import java.util.ArrayList;
import java.util.List;

import javax.swing.JComponent;

/**
 * Abstract base class for property viewers.
 * 
 * @param <T>
 *            Type of property data.
 * @author Peter Hofer
 */
public abstract class AbstractPropertyView<T> extends JComponent implements PropertyView<T> {

	/**
	 * Provider of the currently displayed property data.
	 */
	private Object currentProvider = null;

	private List<PropertySourceListener<T>> sourceListeners = new ArrayList<PropertySourceListener<T>>();

	private void firePropertySourceChanged() {
		for (PropertySourceListener<T> listener : sourceListeners) {
			listener.propertySourceChanged(this, currentProvider);
		}
	}

	/**
	 * Changes the property data displayed by this view.
	 * 
	 * @param properties
	 *            Property data to display.
	 */
	protected abstract void setPropertyData(T properties);

	public void setProperties(Object provider, T properties) {
		setPropertyData(properties);

		if (provider != currentProvider) {
			currentProvider = provider;
			firePropertySourceChanged();
		}
	}

	@Override
	public void addPropertySourceListener(PropertySourceListener<T> listener) {
		sourceListeners.add(listener);
	}
}
