/**
 ** $Id: EventCountFrame.java 22 2005-06-14 05:17:45Z rej $
 ** Copyright (c) 2002 Sun Microsystems, Inc.
 **
 ** See the file "license.terms" for information on usage and redistribution
 ** of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 **/

package gcspy.vis;

import gcspy.interpreter.TraceData;
import gcspy.interpreter.client.EventCommandListener;
import gcspy.vis.utils.Factory;

import java.awt.BorderLayout;

import javax.swing.JComponent;
import javax.swing.JTable;

/**
 * View that displays event counts for a trace.
 * 
 * @author Tony Printezis
 * @author Peter Hofer
 */
class EventCountView extends JComponent {

	private TraceData data;
	private InterpreterEventProvider provider;

	private JTable table;
	private EventCountTableModel model;

	private EventCommandListener eventListener = new EventCommandListener() {

		@Override
		public void onEventCmd(int eventID, int elapsedTime, int compensationTime) {
			int count[] = data.getEventCount();
			model.setData(count);
		}
	};

	/**
	 * Creates an initially empty event count view.
	 */
	EventCountView() {
		this(null, null);
	}

	/**
	 * Creates a new event count view.
	 * 
	 * @param data
	 *            The trace data
	 * @param provider
	 *            Provider for interpreter events.
	 */
	EventCountView(TraceData data, InterpreterEventProvider provider) {
		model = new EventCountTableModel();
		table = Factory.createTable(model);
		table.getColumnModel().getColumn(0).setPreferredWidth(200);

		setLayout(new BorderLayout());
		add(table, BorderLayout.CENTER);
		add(table.getTableHeader(), BorderLayout.NORTH);

		setTrace(data, provider);
	}

	/**
	 * Sets the trace data and event provider for this event count view.
	 * 
	 * @param data
	 *            The trace data
	 * @param provider
	 *            Provider for interpreter events.
	 */
	public void setTrace(TraceData data, InterpreterEventProvider provider) {
		this.data = data;
		model.setTraceData(data);

		if (this.provider != null) {
			this.provider.removeEventListener(eventListener);
		}

		if (provider != null) {
			provider.addEventListener(eventListener);
		}
		this.provider = provider;
	}

}
