/**
 ** $Id: SummaryFrame.java 24 2005-06-17 09:37:56Z rej $
 ** Copyright (c) 2002 Sun Microsystems, Inc.
 **
 ** See the file "license.terms" for information on usage and redistribution
 ** of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 **/

package gcspy.vis;

import gcspy.interpreter.Space;
import gcspy.interpreter.client.SpaceCommandListener;

import java.awt.BorderLayout;
import java.awt.Toolkit;
import java.awt.Window;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.StringSelection;

import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JScrollPane;

/**
 * Displays a summary of the streams of a space.
 * 
 * @author Tony Printezis
 * @author Peter Hofer
 */
class SpaceSummary extends JComponent {

	private JLabel infoL;

	private SpaceSummaryView summaryView;
	private JScrollPane summaryPane;

	private Space space;

	private SpaceCommandListener spaceListener = new SpaceCommandListener() {
		public void onSpaceCmd(Space space) {
			if (SpaceSummary.this.space.getID() == space.getID()) {
				SpaceSummary.this.space = space;
				update();
			}
		}
	};

	private void update() {
		if (isVisible()) {
			infoL.setText(space.getSpaceInfo());
			summaryView.setSpace(space);
			summaryView.update();
		}
	}

	private String getTextSummary() {
		StringBuilder summary = new StringBuilder();

		summary.append(space.getFullName());
		summary.append('\n');

		summary.append(space.getSpaceInfo());
		summary.append('\n');

		summary.append(space.getSpaceString("", "   "));

		return summary.toString();
	}

	public void copyToClipboard() {
		String summary = getTextSummary();
		Clipboard clipboard = Toolkit.getDefaultToolkit().getSystemClipboard();
		clipboard.setContents(new StringSelection(summary), null);
	}

	/**
	 * Create a new summary frame
	 * 
	 * @param owner
	 *            The parent frame
	 * @param interpreter
	 *            The client interpreter
	 * @param space
	 *            The space
	 */
	SpaceSummary(Window owner, InterpreterEventProvider interpreter, Space space) {
		this.space = space;

		setLayout(new BorderLayout());

		summaryView = new SpaceSummaryView(space);
		summaryPane = new JScrollPane(summaryView);
		add(summaryPane, BorderLayout.CENTER);

		infoL = new JLabel();
		add(infoL, BorderLayout.SOUTH);

		interpreter.addSpaceListener(spaceListener);
	}

}
