/**
 ** $Id: ColorChooser.java 28 2005-06-20 13:13:35Z rej $
 ** Copyright (c) 2002 Sun Microsystems, Inc.
 **
 ** See the file "license.terms" for information on usage and redistribution
 ** of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 **/

package gcspy.vis.utils;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Rectangle;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.ArrayList;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.JColorChooser;
import javax.swing.JComponent;

/**
 * Component for choosing colors.
 * 
 * @author Peter Hofer
 */
public class ColorChooser extends JComponent {

	public static final Color DEFAULT_COLOR = Color.WHITE;

	private Color color = DEFAULT_COLOR;

	private MouseListener clickListener = new MouseAdapter() {
		@Override
		public void mouseClicked(MouseEvent e) {
			if (isEnabled()) {
				Color chosen = JColorChooser.showDialog(ColorChooser.this,
					"Select color", getColor());

				if (chosen != null) {
					setColor(chosen);
				}
			}
		}
	};

	private List<ColorChooserListener> listeners = new ArrayList<ColorChooserListener>();

	/**
	 * Add a listener that is notified when the selected color changes.
	 */
	public void addListener(ColorChooserListener listener) {
		listeners.add(listener);
	}

	/**
	 * Sets the selected color.
	 */
	public void setColor(Color color) {
		if (!this.color.equals(color)) {
			this.color = color;
			update();
			fireColorChosen();
		}
	}

	/**
	 * @return the selected color.
	 */
	public Color getColor() {
		return color;
	}

	private void fireColorChosen() {
		for (ColorChooserListener listener : listeners) {
			listener.colorChosen(this, color);
		}
	}

	private void update() {
		repaint();
	}

	@Override
	protected void paintComponent(Graphics g) {
		Rectangle bounds = g.getClipBounds();
		g.setColor(color);
		g.fillRect(bounds.x, bounds.y, bounds.width, bounds.height);
	}

	/**
	 * Constructor for a color chooser with an initially chosen color of
	 * {@link #DEFAULT_COLOR}.
	 */
	public ColorChooser() {
		this(DEFAULT_COLOR);
	}

	/**
	 * Create a color chooser with the specified color as initially chosen
	 * color.
	 * 
	 * @param color
	 *            The initially chosen color
	 */
	public ColorChooser(Color color) {
		if (color == null)
			throw new NullPointerException("color");

		addMouseListener(this.clickListener);
		setBorder(BorderFactory.createEtchedBorder());
		setPreferredSize(new Dimension(30, 15));

		setColor(color);
	}

}
