/**
 ** $Id: IconFactory.java 47 2006-11-22 10:31:24Z rej $
 ** Copyright (c) 2002 Sun Microsystems, Inc.
 **
 ** See the file "license.terms" for information on usage and redistribution
 ** of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 **/

package gcspy.vis.utils;

import java.net.URL;
import java.util.HashMap;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.swing.Icon;
import javax.swing.ImageIcon;

/**
 * Icon Factory for GUI components
 * 
 * @author Tony Printezis
 * @author <a href="http://www/cs/kent.ac.uk">Richard Jones</a>
 * @author Peter Hofer
 */
public class IconLoader {
	private static final Logger logger = Logger.getLogger(IconLoader.class.getName());

	public static final String PLAY_ICON = "play_trans.gif";
	public static final String PLAY_ONE_ICON = "play_one_trans.gif";
	public static final String PAUSE_ICON = "pause_trans.gif";
	public static final String BACK_ONE_ICON = "back_one_trans.gif";
	public static final String REC_ICON = "rec_trans.gif";
	public static final String STOP_ICON = "stop_trans.gif";
	public static final String MAGNIFIER_ICON = "magnifier_trans.gif";
	public static final String MAGNIFIER_LARGER_ICON = "magnifier_larger_trans.gif";
	public static final String MAGNIFIER_SMALLER_ICON = "magnifier_smaller_trans.gif";
	public static final String COPY_ICON = "copy_trans.gif";

	private static final HashMap<String, Icon> cachedIcons = new HashMap<String, Icon>();

	private static final String ICON_PATH = "/icons/";

	/**
	 * Loads the icon with the specified file path or returns a previously
	 * loaded instance that was cached for re-use.
	 * 
	 * @param filepath
	 *            File path of the icon.
	 * @return {@link Icon} instance with the specified image file as content.
	 */
	public static Icon getIcon(String filepath) {
		Icon icon = cachedIcons.get(filepath);
		if (icon == null) {
			URL resource = Object.class.getResource(ICON_PATH + filepath);
			if (resource != null) {
				icon = new ImageIcon(resource);
				cachedIcons.put(filepath, icon);
			} else {
				logger.log(Level.SEVERE, "Could not load icon: maybe you forgot to make install?");
			}
		}
		return icon;
	}
}
